<?php

/*
 *	Function Name	: GeneratePostFields()
 *
 *	Args			: Array .
 *
 *	Returns			: xml
 *
 *	Task			: To Generate Post Fields of Array.
 *****************************************************************************/
function GeneratePostFields($KeyValuePair=array())
{
	$toRet = '';
	foreach ($KeyValuePair as $k => $v) {
		$toRet = $toRet."&".$k."=".$v;
	}
	$toRet = substr($toRet, 1);
	return $toRet;
}
/*
*	GeneratePostFields() method ends here
*****************************************************************************/
/*
*	Function Name	: SGHashHmac()
*
*	Args			: algorithm, data, key, raw output
*
*	Returns			: keyed hash value
*
*	Task			: Generate a keyed hash value
*****************************************************************************/
function SGHashHmac($algo, $data, $key, $rawOutput=false)
{
	$phpVersion = PHP_VERSION;
	if($phpVersion[0] == '5' || function_exists('hash_hmac'))
	{
		return hash_hmac($algo, $data, $key);
	}
	else
	{
		return SGHmacGenerator($algo, $data, $key, $rawOutput);
	}
}
/*
*	SGHashHmac() method ends here
*****************************************************************************/
/*
*	Function Name	: SGHmacGenerator()
*
*	Args			: algorithm, data, key, raw output
*
*	Returns			: keyed hash value
*
*	Task			: Generate a keyed hash value
*****************************************************************************/
function SGHmacGenerator($algo, $data, $key, $rawOutput=false)
{
	//It's only usable with MD5 and SHA1 encryption algorithms - to support PHP4
	$algo = strtolower($algo);
	$pack = 'H'.strlen($algo('test'));
	$size = 64;
	$opad = str_repeat(chr(0x5C), $size);
	$ipad = str_repeat(chr(0x36), $size);

	if (strlen($key) > $size) {
		$key = str_pad(pack($pack, $algo($key)), $size, chr(0x00));
	} else {
		$key = str_pad($key, $size, chr(0x00));
	}

	for ($i = 0; $i < strlen($key) - 1; $i++) {
		$opad[$i] = $opad[$i] ^ $key[$i];
		$ipad[$i] = $ipad[$i] ^ $key[$i];
	}

	$output = $algo($opad.pack($pack, $algo($ipad.$data)));

	return ($rawOutput) ? pack($pack, $output) : $output;
}
/*	SGHashHmac() method ends here
*****************************************************************************/
/*
*	Function Name	: GetXmlValueByTag()
*
*	Args			: xml and tag name
*
*	Returns			: Array
*
*	Task			: Generate a array for the given tag.
*****************************************************************************/
function GetXmlValueByTag($xml, $tag)
{
	$parser = xml_parser_create();
	xml_parse_into_struct($parser, $xml, $arrVals);
	xml_parser_free($parser);
	foreach ($arrVals as $keyArr )
	{
		if($keyArr['tag']==strtoupper($tag))
		{
			$tagValue[] = $keyArr['attributes'];
		}
	}
	return $tagValue;
}
/*
*	GetXmlValueByTag() method ends here
*****************************************************************************/
/*
*	Function Name	: GetResellerList()
*
*	Args			: URL, UserName, Password, StartRowNumber, and ResultSize.
*
*	Returns			: Array
*
*	Task			: Fetch the Reseller List for the Specified Server.
*****************************************************************************/
function GetResellerList($url, $userName, $loginPassword, $from, $resultSize)
{
	$usrLogTime = gmdate('Ymwdhis');
	$usrRequest = 'ListResellers';
	$loginType = 2;
	$sessionID = session_id();
	if($sessionID == '')
	{
		session_start();
		$sessionID = session_id();
	}
	$passwordOne = md5($loginPassword).$usrLogTime;
	$passwordTwo = md5(md5($loginPassword)).$usrLogTime;
	$signatureOne = SGHashHmac('sha1', $usrRequest, $passwordOne);
	$signatureTwo = SGHashHmac('sha1', $usrRequest, $passwordTwo);
	
	
	$myArray = array("UserName"=>$userName, "Signature1"=>$signatureOne, "Signature2"=>$signatureTwo, "SignatureVersion"=>$loginType, "LoginTime"=>$usrLogTime, "Action"=>$usrRequest);

    $toAppend = array("From"=>$from, "ResultSize"=>$resultSize);

    
	$newArray = array_merge($myArray, $toAppend);
	$postFields =  GeneratePostFields($newArray);
	/* Curl Handling */ 
	$curl_handle=curl_init();
	curl_setopt($curl_handle, CURLOPT_URL,$url);
	curl_setopt($curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
	curl_setopt($curl_handle, CURLOPT_POSTFIELDS, $postFields);
	curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER,1);
	$response = curl_exec($curl_handle);
    	echo $response."<br />";
	if(curl_errno($curl_handle)){
		//print curl_error($curl_handle);
	}
	curl_close($curl_handle);
	
	echo htmlentities($response);
		
	/* 											* /
	Parse this Response XML And Use wherever you want
	/ * 										*/
 	$resArr = GetXmlValueByTag(trim($response),'Group');
	return $resArr;
}
/*	GetResellerList() method ends here
*****************************************************************************/
/*
*	Function Name	: GetCustomersList()
*
*	Args			: URL, UserName, Password, StartRowNumber, ResultSize, and Reseller Name.
*
*	Returns			: Array
*
*	Task			: Fetch the Customer List for the specified Reseller 
*****************************************************************************/
function GetCustomersList($url, $userName, $loginPassword, $from, $resultSize, $grpName)
{
	$usrLogTime = gmdate('Ymwdhis');
	$usrRequest = 'ListCustomers';
	$loginType = 2;
	$sessionID = session_id();
	if($sessionID == '')
	{
		session_start();
		$sessionID = session_id();
	}
	$passwordOne = md5($loginPassword).$usrLogTime;
	$passwordTwo = md5(md5($loginPassword)).$usrLogTime;
	$signatureOne = SGHashHmac('sha1', $usrRequest, $passwordOne);
	$signatureTwo = SGHashHmac('sha1', $usrRequest, $passwordTwo);
	
	
	$myArray = array("UserName"=>$userName, "Signature1"=>$signatureOne, "Signature2"=>$signatureTwo, "SignatureVersion"=>$loginType, "LoginTime"=>$usrLogTime, "Action"=>$usrRequest);


    $toAppend = array("From"=>$from, "ResultSize"=>$resultSize, "ResellerName"=>$grpName);
	$newArray = array_merge($myArray, $toAppend);
	$postFields =  GeneratePostFields($newArray);
	
	/* Curl Handling */ 
	$curl_handle=curl_init();
	curl_setopt($curl_handle, CURLOPT_URL,$url);
	curl_setopt($curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
	curl_setopt($curl_handle, CURLOPT_POSTFIELDS, $postFields);
	curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER,1);
	$response = curl_exec($curl_handle);
	if(curl_errno($curl_handle)){
		//print curl_error($curl_handle);
	}
	curl_close($curl_handle);
	echo 'START RESPONSE FROM API<br>';
	echo $response."<br />";
    echo 'END RESPONSE FROM API<br>';
		
	/* 											* /
	Parse this Response XML And Use wherever you want
	/ * 										*/
 	$resArr = GetXmlValueByTag(trim($response),'Group');
	return $resArr;
}
/*
*	GetCustomersList() method ends here
*****************************************************************************/
?>